import 'package:flutter/material.dart';
import '../marker_handler/marker_data.dart';
import 'package:latlong2/latlong.dart' as latLng;

// InteractivePolygon returns a Polygon in a CustomPaint, in a GestureDetector
// in order to make the polygon clickable
class InteractivePolygon extends StatelessWidget {
  final List<latLng.LatLng> points;
  final void Function(Measurement)? onTap;
  final Color color;
  final double strokeWidth;

  InteractivePolygon({
    required this.points,
    required this.onTap,
    this.color = Colors.blue,
    this.strokeWidth = 1.0,
  });

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
      },
      child: CustomPaint(
        painter: PolygonPainter(
          points: points,
          bodyColor: color,
          borderColor: Colors.deepOrange,
          strokeWidth: strokeWidth,
        ),
      ),
    );
  }
}

// PolygonPainter is a custom polygon renderer for Measurement objects
// NB: from https://www.kindacode.com/article/flutter-custompaint-and-custompainter/
class PolygonPainter extends CustomPainter {
  final List<latLng.LatLng> points;
  final Color bodyColor;
  final Color borderColor;
  final double strokeWidth;

  PolygonPainter({
    required this.points,
    required this.bodyColor,
    required this.strokeWidth,
    required this.borderColor,
  });

  // paint() iterates through all the vertices of the polygon and connects
  // each sequential pair with a line
  @override
  void paint(Canvas canvas, Size size) {

    final Paint body = Paint();
    body
      ..color = bodyColor
      ..style = PaintingStyle.fill
      ..strokeWidth = 0;

    final Paint border = Paint();
    border
      ..color = borderColor
      ..style = PaintingStyle.stroke
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 1.0;

    final path = Path();
    if (points.isNotEmpty) {
      print("\n");
      for (final coord in points) {
        print("Corner: ${coord.latitude}, ${coord.longitude}");
      }

      final firstPoint = points.first;
      path.moveTo(firstPoint.latitude, firstPoint.longitude);

      for (var i = 1; i < points.length; i++) {
        final point = points[i];
        path.lineTo(point.latitude, point.longitude);
      }

      path.close();
      canvas.drawPath(path, body); // Render body
      canvas.drawPath(path, border); // Render border
    }
    else {
      print("Error in rendering polygon. No points provided");
    }
  }

  @override
  bool shouldRepaint(PolygonPainter oldDelegate) {
    return oldDelegate.points != points ||
        oldDelegate.bodyColor != bodyColor ||
        oldDelegate.strokeWidth != strokeWidth;
  }
}
